@@ -14,7 +14,7 @@ class detrend():
     Ths class detrends light curves using GPs
     """
 
-    def __init__(self,fname,bin=10,err=0,normalise=False):
+    def __init__(self,fname,bin=10,err=0,normalise=True):
         """Load the light curve to be detrended
         The bin variables are used to speed up computation of the GP
         """
@@ -45,8 +45,8 @@ class detrend():
 
     def add_transits(self,pars,ldc):
         """
-        Add transits, so they can be removed from the detrending routines
-        pars -> [T0, P, a/R*, inclination, Rp/R*] x Number of planets
+        Add transits:
+        pars -> [T0, P, a/R*,b, Rp/R*] x Number of planets
         ldc  -> u1, u2
         """
 
@@ -65,8 +65,9 @@ class detrend():
         flux = 1
         flux_bin = 1
         for i in range(npl):
-            flux     = flux     * tm.evaluate(t0=pars[0+5*i], p=pars[1+5*i], a=pars[2+5*i], i=pars[3+5*i],k=pars[4+5*i], ldc=ldc)
-            flux_bin = flux_bin * tm_bin.evaluate(t0=pars[0+5*i], p=pars[1+5*i], a=pars[2+5*i], i=pars[3+5*i],k=pars[4+5*i], ldc=ldc)
+            incl = np.arccos(pars[3+5*i]/pars[2+5*i])
+            flux     = flux     * tm.evaluate(t0=pars[0+5*i], p=pars[1+5*i], a=pars[2+5*i], i=incl,k=pars[4+5*i], ldc=ldc)
+            flux_bin = flux_bin * tm_bin.evaluate(t0=pars[0+5*i], p=pars[1+5*i], a=pars[2+5*i], i=incl,k=pars[4+5*i], ldc=ldc)
 
         self.flux_planet = flux
         self.flux_planet_bin = flux_bin
@@ -74,7 +75,55 @@ class detrend():
         self.flux_no_planet_bin = self.flux_bin / flux_bin
 
 
+    def mask_transits(self,windows=6./24.):
+        '''
+        Mask the transits for the binned vectors
+        So the GP will work only on out of transit data
+        '''
+
+        self.masked_transits = True
+
+        #Extract the ephemeris from the planet_pars attribute
+        if hasattr(self,'planet_pars'):
+            T0 = self.planet_pars[0::5]
+            P  = self.planet_pars[1::5]
+        else:
+            print("There are no planet parameters in the current class")
+
+
+        if windows.__class__ != list:
+            windows = [windows]*self.nplanets
+        else:
+            if len(windows) != self.nplanets:
+                windows = [max(windows)]*self.nplanets
+
+        #Create a list of lists to find the regions where the transits are for each planet
+        tr = [None]*self.nplanets
+
+        for o in range(0,self.nplanets):
+            phase = ((self.time_bin-T0[o])%P[o])/P[o]
+            phase[phase>0.5] -= 1
+            tr[o] = abs(phase) >= (2*windows[o])/P[o]
+
+        #Let us combine all the data with a logical or
+        indices = tr[0]
+        if self.nplanets > 1:
+            for o in range(1,self.nplanets):
+                indices = np.logical_or(indices,tr[o])
+
+        #Let us store the data with the planets masked out
+        self.time_bin = self.time_bin[indices]
+        self.flux_bin = self.flux_bin[indices]
+        self.ferr_bin = self.ferr_bin[indices]
+        self.flux_planet_bin = self.flux_planet_bin[indices]
+        self.flux_no_planet_bin = self.flux_no_planet_bin[indices]
+
+
     def get_gp(self,Kernel="Exp",amplitude=1e-3,metric=10.,gamma=10.,period=10.):
+        """
+        Citlalicue uses the kernels provided by george, now the options are "Exp", "Matern32", "Matern52", and Quasi-Periodic "QP"
+        User can modify hyper parameters amplitude, metric, gamma, period.
+        """
         import george
         from george import kernels
         if Kernel == "Matern32":
@@ -87,7 +136,6 @@ class detrend():
             log_period = np.log(period)
             kernel = amplitude * kernels.ExpKernel(metric)*kernels.ExpSine2Kernel(gamma,log_period)
 
-
         self.kernel = kernel
         #Compute the kernel with George
         self.gp = george.GP(self.kernel,mean=1)
@@ -108,7 +156,6 @@ class detrend():
         plt.plot(self.time_bin,pred,'r',zorder=2)
         plt.show()
 
-
     #p has to be a vector that contains the hyper parameters
     def neg_ln_like(self,p):
         self.gp.set_parameter_vector(p)
@@ -146,17 +193,37 @@ class detrend():
         else:
             self.result = minimize(self.neg_ln_like,self.gp.get_parameter_vector(),jac=self.grad_neg_ln_like)
 
-    def detrend(self):
-        """detrend the original data set"""
+        #Add the result to our GP object
         #Take the values from the optimisation
         self.gp.set_parameter_vector(self.result.x)
-        #Recompute the correlation matrix
-        self.gp.compute(self.time,self.ferr)
-        #Predict the model for the original data set
-        self.pred, self.pred_var = self.gp.predict(self.flux_no_planet, self.time, return_var=True)
-        #Compute the detrended flux
-        self.flux_detrended = self.flux / self.pred
 
+    def detrend(self,method='interpolation'):
+        """detrend the original data set
+           There are two methods to compute the detrend light curve
+           method = gp, it computes the gp for the whole data set, this is computational demanding
+           method = interpolation, interpolates the GP using the GP from the binned data set, this is faster
+                    but you have to be careful.
+        """
+
+        if method == 'gp':
+            #Recompute the correlation matrix
+            self.gp.compute(self.time,self.ferr)
+            #Predict the model for the original data set
+            self.pred, self.pred_var = self.gp.predict(self.flux_no_planet, self.time, return_var=True)
+            #Compute the detrended flux
+            self.flux_detrended = self.flux / self.pred
+        elif method[0:2] == 'in':
+            from scipy.interpolate import interp1d
+            #Recompute the correlation matrix
+            self.gp.compute(self.time_bin,self.ferr_bin)
+            #Predict the model for the binned data set
+            pred, pred_var = self.gp.predict(self.flux_no_planet_bin, self.time_bin, return_var=True)
+            #Create the interpolation instance
+            f = interp1d(self.time_bin, pred, kind='cubic',fill_value="extrapolate")
+            self.pred = f(self.time)
+            self.flux_detrended = self.flux / self.pred
+
+        #Store the data in a file
         vectorsote = np.array([self.time,self.flux_detrended,self.ferr,self.flux,self.pred,self.flux_planet])
         header = "Time  Detrended_flux  flux_error  flux  GP_model  planets_model"
         fname = self.fname[:-4]+'_detrended.dat'
@@ -210,18 +277,30 @@ class detrend():
         np.savetxt(fname,vectorsote.T)
 
 
-    def plot(self,fsx=15,fsy=5,fname='light_curve.pdf',save=False,show=True,xlim=[None]):
+    def plot(self,fsx=15,fsy=5,fname='light_curve.pdf',save=False,show=True,xlim=[None],show_transit_positions=True,\
+             xlabel='Time [days]',ylabel='Normalised Flux',data_label='LC data',\
+             model_label='Model',detrended_data_label='LC detrended',flat_model_label='Flat LC model'):
         plt.figure(figsize=(fsx,fsy))
-        plt.xlabel('Time [days]')
-        plt.ylabel('Normalised flux')
-        plt.plot(self.time,self.flux,'.',color="#bcbcbc",alpha=0.5,label='LC data')
+        plt.xlabel(xlabel)
+        plt.ylabel(ylabel)
+        plt.plot(self.time,self.flux,'.',color="#bcbcbc",alpha=0.5,label=data_label)
         if hasattr(self,'pred'):
-            plt.plot(self.time,self.pred*self.flux_planet,'-',color="#b30000",label='Model')
+            plt.plot(self.time,self.pred*self.flux_planet,'-',color="#b30000",label=model_label)
         if hasattr(self,'flux_detrended'):
-            plt.plot(self.time,self.flux_detrended-6*np.std(self.flux),'.',color="#005ab3",alpha=0.5,label='LC detrended')
-            plt.plot(self.time,self.flux_planet-6*np.std(self.flux),'#ff7f00',label='Flat LC model')
+            plt.plot(self.time,self.flux_detrended-6*np.std(self.flux),'.',color="#005ab3",alpha=0.5,label=detrended_data_label)
+            plt.plot(self.time,self.flux_planet-6*np.std(self.flux),'#ff7f00',label=flat_model_label)
             plt.ylabel('Normalised flux + offset')
-        plt.legend(loc=1,ncol=5,scatterpoints=1,numpoints=1,frameon=True)
+        if show_transit_positions:
+            if hasattr(self,'planet_pars'):
+                T0 = self.planet_pars[0::5]
+                P  = self.planet_pars[1::5]
+                for i in range(len(T0)):
+                    #where does the first transit happen?
+                    n = int((T0[i] - self.time.min())%P[i])
+                    t0s = np.arange(T0[i] - n*P[i],self.time.max(),P[i])
+                    ys = [max(self.flux)]*len(t0s)
+                    plt.plot(t0s,ys,'v')
+        plt.legend(loc=4,ncol=5,scatterpoints=1,numpoints=1,frameon=True)
         plt.xlim(self.time.min(),self.time.max())
         try:
             plt.xlim(*xlim)
